﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.ConditionActionEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.VariableVector {
  [StorableClass]
  [Item("VariableVectorAction", "")]
  public class VariableVectorAction : Dictionary<string, IActionVariable>, IAction {

    public IOrderedEnumerable<string> Order { get { return this.Keys.OrderBy(x => x); } }

    public int VirtualLength { get { return this.Values.Sum(x => x.VirtualLength); } }

    [StorableConstructor]
    protected VariableVectorAction(bool deserializing) { }
    protected VariableVectorAction(VariableVectorAction original, Cloner cloner) {
      foreach (var item in original) {
        Add((string)item.Key.Clone(), (IActionVariable)item.Value.Clone());
      }
    }
    public VariableVectorAction() : base() { }
    public VariableVectorAction(int capacity) : base(capacity) { }
    public VariableVectorAction(IDictionary<string, IActionVariable> dictionary) : base(dictionary) { }

    //protected override void OnItemsAdded(IEnumerable<KeyValuePair<string, IActionVariable>> items) {
    //  foreach (var item in items) {
    //    if (!(item.Value is StringVariable || item.Value is IntVariable)) {
    //      throw new ArgumentException("Value has to be of type 'StringVariable' or 'IntVariable'");
    //    }
    //  }
    //}

    public VariableVectorAction GetEmptyCopy() {
      VariableVectorAction emptyCopy = new VariableVectorAction(this.Count);
      foreach (var keyValuePair in this) {
        emptyCopy.Add(keyValuePair.Key, keyValuePair.Value.GetEmptyCopy());
      }
      return emptyCopy;
    }

    public VariableVectorAction GetSetCopy() {
      VariableVectorAction setCopy = new VariableVectorAction(this.Count);
      foreach (var keyValuePair in this) {
        setCopy.Add(keyValuePair.Key, keyValuePair.Value.GetSetCopy());
      }
      return setCopy;
    }

    public bool Match(IAction target) {
      var targetCast = target as VariableVectorAction;
      if (targetCast == null) { return false; }
      if (!this.Order.SequenceEqual(targetCast.Order)) { return false; }
      foreach (var keyValuePair in targetCast) {
        if (!this[keyValuePair.Key].MatchVariable(keyValuePair.Value)) {
          return false;
        }
      }
      return true;
    }

    public void Add(IEnumerable<IActionVariable> action) {
      foreach (var variable in action) {
        this.Add(variable.VariableName, variable);
      }
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      bool first = true;
      foreach (var variable in this.Values) {
        if (first) {
          first = false;
        } else {
          sb.Append(";");
        }
        sb.Append(variable.ToString());
      }
      return sb.ToString();
    }

    public void Randomize(IRandom random) {
      foreach (var variable in this.Values) {
        variable.Randomize(random);
      }
    }

    public bool Identical(VariableVectorAction target) {
      if (this.Order.SequenceEqual(target.Order)) { return false; }
      foreach (var keyValuePair in target) {
        if (!this[keyValuePair.Key].Identical(keyValuePair.Value)) {
          return false;
        }
      }
      return true;
    }

    public virtual string ItemName {
      get { return ItemAttribute.GetName(this.GetType()); }
    }
    public virtual string ItemDescription {
      get { return ItemAttribute.GetDescription(this.GetType()); }
    }
    public Version ItemVersion {
      get { return ItemAttribute.GetVersion(this.GetType()); }
    }
    public static Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Class; }
    }
    public virtual Image ItemImage {
      get { return ItemAttribute.GetImage(this.GetType()); }
    }

    public virtual IDeepCloneable Clone(Cloner cloner) {
      return new VariableVectorAction(this, cloner);
    }

    public object Clone() {
      return Clone(new Cloner());
    }

    public event EventHandler ItemImageChanged;
    protected virtual void OnItemImageChanged() {
      EventHandler handler = ItemImageChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }
    public event EventHandler ToStringChanged;
    protected virtual void OnToStringChanged() {
      EventHandler handler = ToStringChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }
  }
}
